<?php

namespace Drupal\site_settings_domain\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\domain\DomainInterface;
use Drupal\site_settings\SiteSettingEntityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

class SiteSettingsDomainController extends ControllerBase {

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * Clones a site setting entity with a different domain context.
   *
   * @param \Drupal\site_settings\SiteSettingEntityInterface $site_setting_entity
   *   The site setting entity to clone.
   * @param \Drupal\domain\DomainInterface|null $domain
   *   The target domain entity.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   The redirect.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function cloneSiteSetting(
    SiteSettingEntityInterface $site_setting_entity,
    ?DomainInterface $domain = NULL,
  ): RedirectResponse {
    $clone = $site_setting_entity->createDuplicate();
    if (isset($domain)) {
      // This is a domain specific one, just save the relevant info.
      $clone->set('domain_context', $domain->id());
      $clone->set('domain_context_parent', $site_setting_entity->id());
      $clone->save();
    }
    else {
      $clone->set('domain_context', NULL);
      $clone->save();

      // Make the fallback the new domain context parent for all linked settings.
      $storage = $this->entityTypeManager->getStorage('site_setting_entity');
      $ids_query = $storage->getQuery()
        ->accessCheck(FALSE);
      $conditions = $ids_query->orConditionGroup()
        ->condition('id', $site_setting_entity->id())
        ->condition('domain_context_parent', $site_setting_entity->id());
      $ids = $ids_query->condition($conditions)->execute();

      foreach ($ids as $id) {
        $linked_setting = $storage->load($id);
        $linked_setting->set('domain_context_parent', $clone->id());
        $linked_setting->save();
      }
    }

    $redirect_url = $clone->toUrl('edit-form');
    return $this->redirect(
      $redirect_url->getRouteName(),
      $redirect_url->getRouteParameters(),
      $redirect_url->getOptions(),
    );
  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function cloneAccess(
    AccountInterface $account,
    SiteSettingEntityInterface $site_setting_entity,
    ?DomainInterface $domain = NULL,
  ): AccessResultInterface {
    $access = $site_setting_entity->access('edit', $account, TRUE);
    if ($domain && $this->moduleHandler()->moduleExists('domain_access')) {
      /** @var \Drupal\domain_access\DomainAccessManagerInterface $access_manager */
      $access_manager = \Drupal::service('domain_access.manager');
      $access->andIf(AccessResult::allowedIf($access_manager->hasDomainPermissions(
        $account,
        $domain,
        ['publish to any assigned domain'],
      )));
    }
    return $access;
  }

}
